@@ -20,6 +20,10 @@
 #include <X11/extensions/Xrender.h>
 #include <X11/Xft/Xft.h>
 
+#ifdef HAVE_CAIRO
+#include <cairo.h>
+#endif
+
 #if RENDER_MAJOR > 0 || RENDER_MINOR >= 5
 #define HAVE_XRENDER
 #endif
@@ -31,8 +35,6 @@
 #ifdef BACKGROUND_IMAGE
 static GdkPixbuf *clock_background, *clock_background_24;
 static guint background_width, background_height;
-#else
-static int border = 2;
 #endif
 
 static GdkPixbuf *day_night_wheel;
@@ -42,12 +44,18 @@ static XftColor color;
 static GtkWidgetClass *parent_class;
 
 #define DEFAULT_RADIUS 64
+#define DEFAULT_BORDER 2
 
 struct _GpeClockFace
 {
   GtkWidget widget;
 
   guint radius;
+  guint border;
+  double hand_width;
+
+  gboolean use_background_image;
+  gboolean label_hours;
 
   guint x_offset, y_offset;
 
@@ -59,6 +67,11 @@ struct _GpeClockFace
   Picture image_pict, src_pict;
 #endif
 
+#ifdef HAVE_CAIRO
+  cairo_t *cr;
+  cairo_surface_t *surface;
+#endif
+
   GdkPixmap *backing_pixmap;
   GdkGC *backing_gc;
   GdkGC *backing_poly_gc;
@@ -87,14 +100,13 @@ typedef struct
 static void
 draw_hand (GpeClockFace *clock,
 	   double angle, 
-	   guint length,
-	   guint thick)
+	   double length,
+	   double thick)
 {
-  GdkPoint points[5];
 #ifdef HAVE_XRENDER
-  XPointDouble poly[5];
+  XPointDouble points[5], poly[5];
 #else
-  FakeXPointDouble poly[5];
+  FakeXPointDouble points[5], poly[5];
 #endif
   int i;
   double sa = sin (angle), ca = cos (angle);
@@ -139,16 +151,18 @@ draw_hand (GpeClockFace *clock,
   else
 #endif
     {
+      GdkPoint gpoints[5];
+
       for (i = 0; i < 5; i++)
 	{
-	  points[i].x = poly[i].x;
-	  points[i].y = poly[i].y;
+	  gpoints[i].x = poly[i].x;
+	  gpoints[i].y = poly[i].y;
 	}
 
       gdk_draw_polygon (clock->backing_pixmap,
 			clock->backing_poly_gc,
 			TRUE,
-			points,
+			gpoints,
 			5);
     }
 }
@@ -174,13 +188,6 @@ gpe_clock_face_expose (GtkWidget *widget,
   int cur_hour;
   gboolean afternoon;
   GdkGC *white_gc;
-#ifdef BACKGROUND_IMAGE
-  GdkRectangle pixbuf_rect, intersect_rect;
-  GdkPixbuf *current_background;
-#else
-  int i;
-  PangoLayout *pl;
-#endif
 
   g_return_val_if_fail (widget != NULL, TRUE);
 
@@ -210,106 +217,137 @@ gpe_clock_face_expose (GtkWidget *widget,
   gdk_gc_set_clip_rectangle (gc, NULL);
   gdk_gc_set_clip_rectangle (white_gc, NULL);
 
-#ifdef BACKGROUND_IMAGE
-  if (afternoon)
-    current_background = clock_background_24;
-  else
-    current_background = clock_background;
-
-  if (event)
+  if (clock->use_background_image)
     {
-      pixbuf_rect.x = clock->x_offset;
-      pixbuf_rect.y = clock->y_offset;
-      pixbuf_rect.width = background_width;
-      pixbuf_rect.height = background_height;
+      GdkRectangle pixbuf_rect, intersect_rect;
+      GdkPixbuf *current_background;
 
-      if (gdk_rectangle_intersect (&pixbuf_rect, &event->area, &intersect_rect) == TRUE)
+      if (afternoon)
+	current_background = clock_background_24;
+      else
+	current_background = clock_background;
+      
+      if (event)
+	{
+	  pixbuf_rect.x = clock->x_offset;
+	  pixbuf_rect.y = clock->y_offset;
+	  pixbuf_rect.width = background_width;
+	  pixbuf_rect.height = background_height;
+	  
+	  if (gdk_rectangle_intersect (&pixbuf_rect, &event->area, &intersect_rect) == TRUE)
+	    gdk_pixbuf_render_to_drawable (current_background, 
+					   clock->backing_pixmap,
+					   clock->backing_gc, 
+					   intersect_rect.x - clock->x_offset, intersect_rect.y - clock->y_offset, 
+					   intersect_rect.x, intersect_rect.y,
+					   intersect_rect.width, intersect_rect.height,
+					   GDK_RGB_DITHER_NONE, 0, 0);
+	}
+      else
 	gdk_pixbuf_render_to_drawable (current_background, 
 				       clock->backing_pixmap,
 				       clock->backing_gc, 
-				       intersect_rect.x - clock->x_offset, intersect_rect.y - clock->y_offset, 
-				       intersect_rect.x, intersect_rect.y,
-				       intersect_rect.width, intersect_rect.height,
+				       0, 0, clock->x_offset, clock->y_offset,
+				       gdk_pixbuf_get_width (clock_background), 
+				       gdk_pixbuf_get_height (clock_background), 
 				       GDK_RGB_DITHER_NONE, 0, 0);
+      
+
+      gdk_pixbuf_render_to_drawable (day_night_wheel, 
+				     clock->backing_pixmap, 
+				     clock->backing_gc, 
+				     0, 0, 
+				     (clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2), 
+				     (clock->y_offset + clock->radius) - (gdk_pixbuf_get_height (day_night_wheel) / 2), 
+				     -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+      
+      gdk_draw_arc (clock->backing_pixmap, 
+		    widget->style->white_gc, TRUE,
+		    ((clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2)) - 2,
+		    ((clock->y_offset + clock->radius) - (gdk_pixbuf_get_height (day_night_wheel) / 2)) - 2,
+		    gdk_pixbuf_get_width (day_night_wheel) + 4,
+		    gdk_pixbuf_get_height (day_night_wheel) + 4,
+		    (gtk_adjustment_get_value (clock->hour_adj) * -360 / 24) * 64
+		    + gtk_adjustment_get_value (clock->minute_adj) * -(360 * 64 / 24) / 60,
+		    180 * 64);
     }
   else
-    gdk_pixbuf_render_to_drawable (current_background, 
-				   clock->backing_pixmap,
-				   clock->backing_gc, 
-				   0, 0, clock->x_offset, clock->y_offset,
-				   gdk_pixbuf_get_width (clock_background), 
-				   gdk_pixbuf_get_height (clock_background), 
-				   GDK_RGB_DITHER_NONE, 0, 0);
-
-#else
-  pl = gtk_widget_create_pango_layout (widget, "");
-
-  gdk_draw_arc (clock->backing_pixmap, gc, TRUE,
-		clock->x_offset, clock->y_offset,
-		clock->radius * 2, clock->radius * 2,
-		0, 360 * 64);
-  
-  gdk_draw_arc (clock->backing_pixmap, white_gc, TRUE,
-                clock->x_offset + border, clock->y_offset + border,
-                (clock->radius - border) * 2, (clock->radius - border) * 2,
-                0, 360 * 64);
-
-  for (i = 0; i < 12; i++)
     {
-      double angle, dx, dy;
-      int x, y, width, height;
-      char buf[3];
-
-      angle = 2 * M_PI * i / 12;
-      
-      dx = sin (angle) * (clock->radius - border - 10);
-      dy = -cos (angle) * (clock->radius - border - 10);
-
-      x = clock->x_offset + clock->radius + dx;
-      y = clock->y_offset + clock->radius + dy;
-
-      sprintf (buf, "%d", afternoon ? (i + 12) : i);
-
-      pango_layout_set_text (pl, buf, strlen (buf));
-      pango_layout_get_size (pl, &width, &height);
-      width /= PANGO_SCALE;
-      height /= PANGO_SCALE;
-
-      gtk_paint_layout (widget->style,
-			clock->backing_pixmap,
-			GTK_WIDGET_STATE (widget),
-			FALSE,
-			NULL,
-			widget,
-			"label",
-			x - (width / 2), y - (height / 2), pl);
-    }
-
-  g_object_unref (pl);
+#ifdef HAVE_CAIRO
+      cairo_set_rgb_color (clock->cr, 1, 1, 1);
+      cairo_arc (clock->cr,
+		 clock->x_offset + clock->radius,
+		 clock->y_offset + clock->radius,
+		 clock->radius,
+		 0,
+		 2 * M_PI);
+      cairo_fill (clock->cr);
+      cairo_set_rgb_color (clock->cr, 0, 0, 0);
+      cairo_arc (clock->cr,
+		 clock->x_offset + clock->radius,
+		 clock->y_offset + clock->radius,
+		 clock->radius,
+		 0,
+		 2 * M_PI);
+      cairo_stroke (clock->cr);
+#else
+      gdk_draw_arc (clock->backing_pixmap, gc, TRUE,
+		    clock->x_offset, clock->y_offset,
+		    clock->radius * 2, clock->radius * 2,
+		    0, 360 * 64);
+  
+      gdk_draw_arc (clock->backing_pixmap, white_gc, TRUE,
+		    clock->x_offset + clock->border, clock->y_offset + clock->border,
+		    (clock->radius - clock->border) * 2, (clock->radius - clock->border) * 2,
+		    0, 360 * 64);
 #endif
 
-  gdk_pixbuf_render_to_drawable (day_night_wheel, 
-				 clock->backing_pixmap, 
-				 clock->backing_gc, 
-				 0, 0, 
-				 (clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2), 
-				 (clock->y_offset + clock->radius) - (gdk_pixbuf_get_height (day_night_wheel) / 2), 
-				 -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
+      if (clock->label_hours)
+	{
+	  PangoLayout *pl;
+	  int i;
+  
+	  pl = gtk_widget_create_pango_layout (widget, "");
 
-  gdk_draw_arc (clock->backing_pixmap, 
-		widget->style->white_gc, TRUE,
-		((clock->x_offset + clock->radius) - (gdk_pixbuf_get_width (day_night_wheel) / 2)) - 2,
-		((clock->y_offset + clock->radius) - (gdk_pixbuf_get_height (day_night_wheel) / 2)) - 2,
-		gdk_pixbuf_get_width (day_night_wheel) + 4,
-		gdk_pixbuf_get_height (day_night_wheel) + 4,
-		(gtk_adjustment_get_value (clock->hour_adj) * -360 / 24) * 64
-		+ gtk_adjustment_get_value (clock->minute_adj) * -(360 * 64 / 24) / 60,
-		180 * 64);
+	  for (i = 0; i < 12; i++)
+	    {
+	      double angle, dx, dy;
+	      int x, y, width, height;
+	      char buf[3];
+	      
+	      angle = 2 * M_PI * i / 12;
+	      
+	      dx = sin (angle) * (clock->radius - clock->border - 10);
+	      dy = -cos (angle) * (clock->radius - clock->border - 10);
+
+	      x = clock->x_offset + clock->radius + dx;
+	      y = clock->y_offset + clock->radius + dy;
+
+	      sprintf (buf, "%d", afternoon ? (i + 12) : i);
+
+	      pango_layout_set_text (pl, buf, strlen (buf));
+	      pango_layout_get_size (pl, &width, &height);
+	      width /= PANGO_SCALE;
+	      height /= PANGO_SCALE;
+
+	      gtk_paint_layout (widget->style,
+				clock->backing_pixmap,
+				GTK_WIDGET_STATE (widget),
+				FALSE,
+				NULL,
+				widget,
+				"label",
+				x - (width / 2), y - (height / 2), pl);
+	    }
+	  
+	  g_object_unref (pl);
+	}
+    }
 
   if (clock->second_adj != NULL)
     draw_hand (clock, clock->second_angle, 7 * clock->radius / 8, 1);
-  draw_hand (clock, clock->minute_angle, 7 * clock->radius / 8, 3);
-  draw_hand (clock, clock->hour_angle, 3 * clock->radius / 5, 3);
+  draw_hand (clock, clock->minute_angle, 7 * clock->radius / 8, clock->hand_width);
+  draw_hand (clock, clock->hour_angle, 3 * clock->radius / 5, clock->hand_width);
 
   gdk_draw_drawable (drawable, gc, clock->backing_pixmap, 0, 0, 0, 0, 
 		     widget->allocation.width, widget->allocation.height);
@@ -452,6 +490,9 @@ gpe_clock_face_prepare_xrender (GtkWidget *widget)
 {
   GpeClockFace *clock = GPE_CLOCK_FACE (widget);
   GdkGCValues gcv;
+  Display *dpy;
+  GdkVisual *gv;
+  GdkColormap *gcm;
 
   clock->backing_pixmap = gdk_pixmap_new (widget->window,
 					  widget->allocation.width, 
@@ -462,18 +503,16 @@ gpe_clock_face_prepare_xrender (GtkWidget *widget)
   gdk_gc_get_values (widget->style->bg_gc[GTK_STATE_NORMAL], &gcv);
   gdk_gc_set_foreground (clock->backing_gc, &gcv.foreground);
 
+  dpy = GDK_WINDOW_XDISPLAY (widget->window);
+  gv = gdk_window_get_visual (widget->window);
+  gcm = gdk_drawable_get_colormap (widget->window);
+      
+
 #ifdef HAVE_XRENDER
   if (clock->can_render)
     {
       XRenderPictureAttributes att;
-      Display *dpy;
-      GdkVisual *gv;
-      GdkColormap *gcm;
 
-      dpy = GDK_WINDOW_XDISPLAY (widget->window);
-      gv = gdk_window_get_visual (widget->window);
-      gcm = gdk_drawable_get_colormap (widget->window);
-      
       clock->draw = XftDrawCreate (dpy, GDK_WINDOW_XWINDOW (clock->backing_pixmap),
 				   gdk_x11_visual_get_xvisual (gv),
 				   gdk_x11_colormap_get_xcolormap (gcm));
@@ -489,6 +528,14 @@ gpe_clock_face_prepare_xrender (GtkWidget *widget)
     {
       clock->backing_poly_gc = gdk_gc_new (clock->backing_pixmap);
     }
+
+#ifdef HAVE_CAIRO
+  clock->cr = cairo_create ();
+  clock->surface = cairo_xlib_surface_create (dpy, GDK_WINDOW_XWINDOW (clock->backing_pixmap), 
+					      gdk_x11_visual_get_xvisual (gv), 0, 
+					      gdk_x11_colormap_get_xcolormap (gcm));
+  cairo_set_target_surface (clock->cr, clock->surface);
+#endif
 }
 
 static void
@@ -681,6 +728,10 @@ gpe_clock_face_init (GpeClockFace *clock)
   gtk_widget_set_double_buffered (GTK_WIDGET (clock), FALSE);
 
   clock->radius = DEFAULT_RADIUS;
+  clock->border = DEFAULT_BORDER;
+  clock->use_background_image = TRUE;
+  clock->label_hours = FALSE;
+  clock->hand_width = 3;
 
 #ifdef HAVE_XRENDER
   clock->draw = NULL;
@@ -723,6 +774,32 @@ gpe_clock_face_set_do_grabs (GpeClockFace *clock, gboolean yes)
   clock->do_grabs = yes;
 }
 
+void
+gpe_clock_face_set_radius (GpeClockFace *clock, guint radius)
+{
+  clock->radius = radius;
+  if (clock->radius != DEFAULT_RADIUS)
+    clock->use_background_image = FALSE;
+}
+
+void
+gpe_clock_face_set_use_background_image (GpeClockFace *clock, gboolean yes)
+{
+  clock->use_background_image = yes;
+}
+
+void
+gpe_clock_face_set_label_hours (GpeClockFace *clock, gboolean yes)
+{
+  clock->label_hours = yes;
+}
+
+void
+gpe_clock_face_set_hand_width (GpeClockFace *clock, double width)
+{
+  clock->hand_width = width;
+}
+
 GType
 gpe_clock_face_get_type (void)
 {
